#include "path.h"
#include "stat.h"
#include "stat_private.h"

using namespace std;

namespace Msp {
namespace FS {

Stat::Stat(const Stat &other):
	exists(other.exists),
	type(other.type),
	size(other.type),
	alloc_size(other.alloc_size),
	mtime(other.mtime),
	owner_info(other.owner_info),
	priv(other.priv ? new Private(*other.priv) : nullptr)
{ }

Stat &Stat::operator=(const Stat &other)
{
	if(&other==this)
		return *this;

	exists = other.exists;
	type = other.type;
	size = other.size;
	alloc_size = other.alloc_size;
	mtime = other.mtime;
	owner_info = other.owner_info;
	delete priv;
	priv = (other.priv ? new Private(*other.priv) : nullptr);

	return *this;
}

Stat::~Stat()
{
	delete priv;
}

const string &Stat::get_owner() const
{
	if(priv && owner_info.owner.empty())
		priv->fill_owner_info(owner_info);
	return owner_info.owner;
}

const string &Stat::get_group() const
{
	if(priv && owner_info.group.empty())
		priv->fill_owner_info(owner_info);
	return owner_info.group;
}

} // namespace FS
} // namespace Msp
