#ifndef MSP_IO_ASSET_H_
#define MSP_IO_ASSET_H_

#include <msp/core/mspcore_api.h>
#include "seekable.h"

namespace Msp {
namespace IO {

/**
Opens a file from the application's assets.  On Android, this means the assets
contained within the APK.  On other platfoms, assets are located in the
directory indicated by FS::get_sys_data_dir().  Assets are always read-only.
*/
class MSPCORE_API Asset: public Seekable
{
private:
	struct Private;

	Private *priv = nullptr;

public:
	Asset(const std::string &);
	~Asset();

	void set_block(bool) override;
	void set_inherit(bool) override;

private:
	std::size_t do_write(const char *, std::size_t) override;
	std::size_t do_read(char *, std::size_t) override;

public:
	const Handle &get_handle(Mode) override;

	SeekOffset seek(SeekOffset, SeekType) override;
	SeekOffset tell() const override;
};

} // namespace IO
} // namespace Msp

#endif
