#ifndef MSP_IO_SERIAL_H_
#define MSP_IO_SERIAL_H_

#include <msp/core/mspcore_api.h>
#include "eventobject.h"
#include "eventreader.h"
#include "handle.h"

namespace Msp {
namespace IO {

class MSPCORE_API Serial: public EventObject
{
public:
	enum Parity
	{
		NONE,
		EVEN,
		ODD
	};

private:
	struct DeviceState;

	Handle handle;
	EventReader reader;

public:
	Serial(const std::string &);
private:
	void platform_init(const std::string &);
public:
	~Serial() override;

private:
	void close();

public:
	void set_block(bool) override;
	void set_inherit(bool) override;

	void set_baud_rate(unsigned);
	void set_data_bits(unsigned);
	void set_parity(Parity);
	void set_stop_bits(unsigned);
	void set_parameters(const std::string &);

private:
	std::size_t do_write(const char *, std::size_t) override;
	std::size_t do_read(char *, std::size_t) override;

public:
	const Handle &get_handle(Mode) override;
	const Handle &get_event_handle() override { return reader.get_event(); }
};

} // namespace IO
} // namespace Msp

#endif
