#ifndef MSP_STRINGCODEC_JISX0208_H_
#define MSP_STRINGCODEC_JISX0208_H_

#include <msp/core/mspcore_api.h>
#include "codec.h"

namespace Msp {
namespace StringCodec {

/**
Codec for the JIS X 0208 encoding.  This is not particularly useful as a
stand-alone codec, due to lack of a linefeed character among other things,
but is included as part of some other encodings.
*/
class MSPCORE_API JisX0208: public StandardCodec<JisX0208>
{
public:
	class MSPCORE_API Encoder: public Codec::Encoder
	{
	public:
		Encoder(ErrorMode em = DEFAULT): Codec::Encoder(em) { }

		void encode_char(unichar, std::string &) override;
	private:
		void transliterate(unichar, std::string &) override;
	};

	class MSPCORE_API Decoder: public Codec::Decoder
	{
	public:
		Decoder(ErrorMode em = DEFAULT): Codec::Decoder(em) { }

		unichar decode_char(const std::string &, std::string::const_iterator &) override;
	};

	JisX0208(ErrorMode em = DEFAULT): StandardCodec<JisX0208>(em) { }

	const char *get_name() const override { return "JIS X 0208"; }
};


struct Kuten
{
	unsigned short ku = 0;
	unsigned short ten = 0;

	explicit operator bool() { return ku!=0 && ten!=0; }
};

MSPCORE_API unichar jisx0208_to_ucs(Kuten);
MSPCORE_API Kuten ucs_to_jisx0208(unichar);

} // namespace StringCodec
} // namespace Msp

#endif
